;------------------------------------------------------------------------------
; Projeto: Programador EPROM Z80 
; Arquivo: progr_eprom.asm
; Desenv : Eng. Márcio José Soares
;
; Permite ler/gravar/verificar uma EPROM usando a placa Z80 1990
; Usa a PPI8255
;
; Montador: pasmo -d -v prog_eprom.asm prog_eprom.bin > prog_eprom.lst
;
; Portas:
;-----------------------------------
; PORTA    ESCRITA       LEITURA
;  A       LSB-MSB ADD
;  B       Controle
;  C       Dados         Dados
;-----------------------------------
;
;------------------------------------------------------------------------------
;
; Últimas alterações:
;	em 06/03/2024:
;		- inicio desse projeto
;		- teste das telas ok! simulação da leitura de 255 bytes
;
;	em 11/03/2024:
;		- testado leitura da EPROM M2732 (4096B) no hardware: OK!!!
;
; 	em 15/03/2024:
;		- preparado tipo de controle para pino OE/VPP junto devido a
;         alteração no hardware
;
;	em 16/03/2024:
;		- inserido menu para seleção da EPROM: 2732, 2764 e 27128
;		- inserido menu para seleção das opções: Ler, verificar, comparar e gravar
;		- inserido "perfumarias" para mostrar qual EPROM está utilizando
;		- alterada função read_eprom para que a mesma use os pinos OE/VPP (2732)
;         e OE (2764 e 27128) de acordo com o tipo da EPROM selecionada
;		- criada função de verificação da EPROM de acordo com tipo selecionado
;		- inseridas mensagens extras para tela
;		- testes de verificação e leitura oks com 2732
;
;	em 19/03/2024:
;		- placa Z80 1990 agora tem 32k re RAM o que possibilita receber o 
;		  arquivo grudado no programa para testar ou ainda utilizando a
;		  subrotina XMODEM pode ser possível receber um arquivo e colocá-lo
;         numa posição "x" da memória!!!
;
;	em 20/03/2024:
;		- testado opção de verificação com EPROM não vazia e EPROM vazia.
;		  Testes ok!!! verificação ok em ambos os casos!
;		- inserida subrotina de gravação para EPROM
;		- posição para ponteiro foi direcionada para final do programa
;		  assim não importa o quanto o mesmo seja alterado, o ponteiro
;         ficará sempre no final
;		- realizados testes do ponteiro - leitura da EPROM insere dados
;         na posição correta!
;		- alterada rotina de gravação para ler dados a partir do ponteiro 
;         na memória
;-		- iniciado teste de gravação mas aparentemente não está gravando
; 		  Preciso comprar pelo menos duas memórias novas para testar
;
;	em 21/03/2024:
;		- alterei o circuito inserindo um regulador de 6V para alimentar
;		  a EPROM quando em modo gravação. Continua não funcionando. Estou 
;		  achando que é problema na EPROM. Preciso de uma nova para testar!
;		- inserida função de comparação dos dados presentes no Buffer com
;         o presente na EPROM.
;
;	em 24/03/2024:
;		- adquiri EPROMs novas e funcionou perfeitamente tanto escrita,
;         leitura e comparação
;
;	em 18/09/2024:
;		- testes com EPROM's 2732 na placa ok!!!
;		  Leitura, comparação, gravação e verificação OK
;		- testes com EPROM's 2764 na placa ok!!!
;		  Leitura, comparação, gravação e verificação OK
;
;	em 19/09/2024:
;		- testes com EPROM's 27128 na placa ok!!!
;		  Leitura, comparação, gravação e verificação OK
;
;	a fazer:
;       - Apresentar projeto!
;		- Futuro: seria legal ter um jeito de ler um arquivo binário do PC
;		  a ser inserido no buffer para posterior gravação - sem necessidade
;         de uma eprom matriz
;
;------------------------------------------------------------------------------

;-------------------------------------------------------------------------------
; Incluí constantes e variáveis para uso dos recursos do programa monitor 
; liberados para o usuário - novo a partir de 22/05/2023
;
include "../drZ80_uC.inc"

;-------------------------------------------------------------------------------
; Definições gerais do módulo
;
LF				equ		0AH				; Line feed
CR				equ		0DH				; Enter
BS_				equ		08H				; backspace/DEL

EN_MSBOFF		equ		0FEH			; pino EN_MSB on
EN_MSBON		equ		01H				; pino EN_MSB 0ff

EN_CEON			equ		02H				; pino EN_CE on
EN_CEOFF		equ		0FDH			; pino EN_CE off

EN_PGMON		equ		04H				; pino EN_PGM on
EN_PGMOFF		equ		0FBH			; pino EN_PGM off

EN_OEVPPON		equ		10H				; pino EN_OEVPP on 
EN_OEVPPOFF		equ		0EFH			; pino EN_OEVPP off

EN_OEON			equ		08H				; pino EN_OE on
EN_OEOFF		equ		0F7H			; pino EN_OE off

CFG8255R        equ     0089H           ;A e B OUT, C IN           
CFG8255W        equ     0080H           ;A, B e C OUT           

T2732			equ		10H				; 1000H = 4096d
T2764			equ		20H				; 2000H = 8192d
T27128			equ		40H				; 4000H = 16384d

;-------------------------------------------------------------------------------
; Declara posição inicial da RAM - inicio do programa do usuário
;
                org     8000H           ; endereço inicial da RAM e programa do usuário
                jr      main            ; desvia para main

;-------------------------------------------------------------------------------
; Declara posição para atendimento de ints do usuário!
; Novo! Apenas para a versão drZ80 v2.0 de 22/05/2023
;
				org		8004H			; aqui tratamento da int do teclado - sempre!!!!
                di						; desliga INTs
                jp		trata_int		; desvia para realizar o tratamento
                
                org		8008H			; aqui tratamento da int NMI para RS232 - sempre!!!!
                di						; desliga INTs
                jp		trata_intNMI	; desvia para realizar o tratamento
                
;-------------------------------------------------------------------------------
; Subrotina principal
;				
main:			call	wrzerodisp		; zera displays
				
				ld		hl,aux			; pega end da var
				ld		a,00H			; zera var
				ld		(hl),a
				
				ld		hl,aux2			; pega end da var
				ld		(hl),a			; zera var
				
				ld		hl,aux3			; pega end da var
				ld		(hl),a			; zera var
				
				ld		hl,aux4			; pega end da var
				ld		(hl),a			; zera var
				
				ld		hl,menu			; pega end da var
				ld		(hl),a			; zera var
				
				ld		hl,epromtype	; pega end da var
				ld		(hl),a			; zera var
				
				ld		hl,mydataRX		; pega end da var
				ld		(hl),a			; zera var
				
				ld		hl,statusB		; pega end da var
				ld		a, 00H			; zera a var
				and		EN_MSBOFF		; pino desligado em 0

				or		EN_OEVPPON		; transistor em condução, pino OE/VPP em ground
				ld		(hl),a			; salva
				out		(_8255_OB),a	; port B com controles desligados
				
				ld		a,00H			; limpa port A 
				out		(_8255_OA),a
				
				call	t_500ms
				call	t_500ms			; aguarda 1s
				
				ld		hl,msginit		; envia mensagem inicial
				call	wrmonRS232		; envia via RS-232
				
				call	readmenuEPR		; desvia para ler tipo de eprom
				
				ld		hl,menu			; pega end da var
				ld		a,(hl)			; conteúdo em A
				cp		01H				; compara
				jr		z,main_seg		; desvia se ok

;				cp		02H				; compara com 2
;				jr		z,main_seg		; desvia se ok

;				cp		03H				; compara com 3
;				jr		z,main_seg		; desvia se ok

				jr		nz,main_lc		; desvia se não pegou nada!
				
main_seg:		call	readmenuOP		; desvia para ler opção do usuário
				
main_lc:		call	t_500ms			; não faz nada
				jr		main_lc			; apenas trava aqui
				
				halt					; mas se passar, trava!

;-------------------------------------------------------------------------------
; Subrotina para tratamento da INT ligada ao teclado
; inicio da subrotina dentro do programa monitor!!!
; aqui apenas tratamento e fechamento da int - retorno!
;
trata_int:		ld		hl,mytec		; pega var onde estara o valo da tecla
				ld      a,(hl)			; coloca em A
				
endint:         exx                      ; restaura registradores
                ex      af,af'           ;
                
                ei                       ; habilita novamente a int
                reti                     ; retorna da int
		
;-------------------------------------------------------------------------------
; Subrotina para tratamento da NMI ligada ao 8250
; inicio da subrotina dentro do programa monitor!!!
; aqui apenas tratamento e fechamento da int - retorno!
;
trata_intNMI:	ld      bc,mydataRX		; pega end da var
				ld      a,(bc)			; pega o conteúdo
				ld      d,a             ; salva em d
				
trata_NMIEND:   exx						; restaura regs salvos pelo
				ex 		af,af'			; monitor na entrada da int
				
				ei						; habilita novamente as ints
				retn      				; retorna da nmi
	
;------------------------------------------------------------------------------
; Subrotina para ler a opção do usuário no menu opções
; 
readmenuOP:		ld		hl,msgCRLF
				call	wrmonRS232
				ld		hl,msgCRLF
				call	wrmonRS232
				
				ld		hl,msgmenuop	; envia opções para usuário
				call	wrmonRS232		; envia novo menu
				
				call	printEPROM		; imprime o tipo da eprom
				
				ld		hl,msgmenuop2
				call	wrmonRS232
				
				ld		hl,msgmenuR		; pede opção
				call	wrmonRS232
				
				ld		hl,mydataRX		; pega end da var
				ld		a,00H
				ld		(hl),a
				
readOPlc:		ld		hl,mydataRX		; pega end da var
				ld		a,(hl)			; pega o conteúdo
				
				cp		00H				; compara com 0
				jr		nz,ehread		; tem algo na var
				call	t_50ms			; aguarda
				jr		readOPlc		; desvia até conseguir algo
				
ehread:			cp		31H				; compara com 1
				jr		nz,ehverif		; desvia para próximo
				call	read_eprom		; desvia para atender
				jr		readmenuOP		; continua até ser o fim
				
ehverif:		cp		32H				; compara com 2
				jr		nz,ehcompar		; desvia para próximo
				call	verif_eprom		; desvia para atender
				jr		readmenuOP		; continua até ser o fim
				
ehcompar:		cp		33H				; compara com 3
				jr		nz,ehwrite		; desvia para próximo
				call	compar_eprom	; desvia para atender
				jr		readmenuOP		; continua até ser o fim
				
ehwrite:		cp		34H				; compara com 4
				jr		nz,ehvoltar		; desvia para próximo
				call	write_eprom		; desvia para atender
				jr		readmenuOP		; continua até ser o fim
				
ehvoltar:		cp		35H				; compara com 5
				jr		nz,eherroOP		; desvia para próximo
				ret						; retorna

eherroOP:		ld		hl,msgerroOp	; avisa erro de opção
				call	wrmonRS232		; envia para tela
				jr		readmenuOP		; desvia e continua
				
;------------------------------------------------------------------------------
; Subrotina para ler a opção do usuário no menu do tipo da EPROM
; 
readmenuEPR:	ld		hl,msgCRLF
				call	wrmonRS232
				ld		hl,msgmenuEPROM
				call	wrmonRS232
				ld		hl,msgmenuR
				call	wrmonRS232

readmenuEPRC:	ld		hl,mydataRX		; pega end da var
				ld		a,(hl)			; pega o conteúdo
				
				cp		00H				; compara com 0
				jr		nz,eh2732		; tem algo na var
				call	t_50ms			; aguarda
				jr		readmenuEPRC	; desvia até conseguir algo
				
eh2732:			cp		31H				; compara com 1
				jr		nz,eh2764		; não é 2732
				ld		b,a				; salva
				jr		readmEPRok		; desvia para o fim
				
eh2764:			cp		32H				; compara com 2
				jr		nz,eh27128		; não é 2764
				ld		b,a				; salva
				jr		readmEPRok		; desvia para o fim
				
eh27128:		cp		33H				; compara com 3
				jr		nz,eherroEP		; não é 27128
				ld		b,a				; salva
				jr		readmEPRok		; desvia para o fim
				
eherroEP:		ld		hl,msgepinv		; avisa erro
				call	wrmonRS232		; envia
				ld		hl,msgCRLF
				call	wrmonRS232
				ld		hl,mydataRX		; pega end da var
				ld		a,00H			; zera var
				ld		(hl),a
				jr		readmenuEPR		; desvia
				
readmEPRok:		ld		hl,menu			; pega end da var
				ld		a,01H			; avisa que atendeu menu1
				ld		(hl),a
				
				ld		a,b				; restaura
				and		03H				; limpa A para manter apenas LSB
				ld		hl,epromtype	; pega end da var
				ld		(hl),a			; salva
				
				ld		hl,mydataRX		; pega end da var
				ld		a,00H			; zera var
				ld		(hl),a
				
				ret						; retorna

;------------------------------------------------------------------------------
; Subrotina para ler uma EPROM
; coloca end nos displays 1 a 4 e dado nos disps de 5 a 6
; também envia informação para tela
;
read_eprom:		ld		hl,msgCRLF		; insere msg sobre o que vai fazer
				call	wrmonRS232		; envia duas vezes
				ld		hl,msgCRLF		; insere msg sobre o que vai fazer
				call	wrmonRS232
				
				ld		hl,msgread
				call	wrmonRS232
				
				call	printEPROM		; envia tipo da eprom
				
				;ld		hl,msgread2		; envia continuação da tela read
				;call	wrmonRS232
				
				ld		hl,msgCRLF		; endereço e dados na linha debaixo
				call	wrmonRS232
				
				ld		a,CFG8255R		; prepara para "ler" dados
				call	cfg_8255		; desvia para reprogramar o 8255
				
				ld		a,01H			; prepara porta de controle
				or		EN_OEVPPON		; transistor em condução, pino OE/VPP em ground
				ld		hl,statusB		; pega end da var
				ld		(hl),a			; guarda status
				out		(_8255_OB),a	; port B com controles desligados
							
				ld		d,00H			; carrega DE com o endereço
				ld		e,00H			
				ld		bc,PONTEPROM	; carrega o endereço do ponteiro
				
read_bytes:		ld		a,d				; d contem MSB do endereço
				out		(_8255_OA),a	; carrega com MSB do endereço

				ld		a,(hl)			; pega o status
				or		EN_MSBON		; faz um or para ligar apenas o pino EN_MSB
				ld		(hl),a			; guarda status
				out		(_8255_OB),a	; transfere MSB do endereço
				
				ld		a,(hl)			; pega o status
				and		EN_MSBOFF		; desliga pino - MSB na posição
				ld		(hl),a			; guarda status
				out		(_8255_OB),a	; desliga pino
				
				ld		a,e				; carrega LSB do endereço
				out		(_8255_OA),a	; carrega com MSB do endereço
				
				ld		a,(hl)			; pega o status
				or		EN_CEON			; pino EN_CE on
				ld		(hl),a			; guarda status
				out		(_8255_OB),a	; liga pino
				
				nop						; aguarda pino estabilizar
				nop
				nop
				nop
				
				push	bc				; salva regs
				push    hl				; 
				ld		hl,epromtype	; pega end da var
				ld		a,(hl)			; conteúdo em A
				cp		01H				; compara com 1
				jr		nz,r64_128on	; desvia
				ld		b,EN_OEVPPON	; carrega b com pino
				jr		contreadON

r64_128on:		ld		b,EN_OEON		; carrega b com pino
contreadON:		pop		hl				; restaura regs
				
				ld		a,(hl)			; pega status
				or		b				; pino EN_OEVPP ou EN_OE
				pop		bc				; restaura reg
				ld		(hl),a			; guarda status
				out		(_8255_OB),a	; liga pino
				
				nop						; aguarda dado estabilizar
				nop
				nop
				nop
				
				in		a,(_8255_OC)	; pega dado
				ld		(bc),a			; salva dado na RAM
				
				push	bc				; salva regs
				push    hl				; 
				ld		hl,epromtype	; pega end da var
				ld		a,(hl)			; conteúdo em A
				cp		01H				; compara com 1
				jr		nz,r64_128off	; desvia
				ld		b,EN_OEVPPOFF	; carrega b com pino
				jr		contreadOFF

r64_128off:		ld		b,EN_OEOFF		; carrega b com pino
contreadOFF:	pop		hl				; restaura reg
				ld		a,(hl)			; pega o status
				and		b				; pino EN_OE ou EN_OEVPP off
				pop		bc				; restaura reg
				and		EN_CEOFF		; pino EN_CE off
				ld		(hl),a			; guarda status
				out		(_8255_OB),a	; desliga pinos CE e OE
				
				;ld		a,(bc)			; dado agora em A
				;call	showdataD		; mostra dados no display
				;SHOW END

				push	bc
				ld		c,BS_
				ld		b,09H
				call	sendCharX		; envia 5 BS_
				pop		bc
				
				ld		a,d				; mostra endereço
				call	bin2hex
				ld		a,e
				call	bin2hex
				
				ld		hl,mydataTX		; pega end da var
				ld		a,'H'
				ld		(hl),a			; salva na var
				call	sndchRS232		; envia char
				
				ld		hl,mydataTX		; pega end da var
				ld		a,' '
				ld		(hl),a			; salva na var
				call	sndchRS232		; envia char
				
				ld		a,(bc)			; pega dado
				call	bin2hex			; mostra na tela
				ld		hl,mydataTX		; pega end da var
				ld		a,'H'
				ld		(hl),a			; salva na var
				call	sndchRS232		; envia char
				
				;ld		a,(bc)			; pega o dado
				;call	showdataD		; mostra dado no display
				
				;ld		a,(bc)
				;call	showdataS		; e via porta serial
								
				ld		hl,statusB		; pega end da var
				
				inc		de				; incrementa endereço
				inc		bc
				push	hl				; salva
				ld		hl,epromtype	; prepara para comparar
				ld		a,(hl)			; conteúdo em A
				pop		hl				; restaura
eh01H:			cp		01H				; compara com 1
				jr		nz,eh02H		; diferente compara outro
eherroH:		ld		a,T2732			; carrega tamanho a comparar
				jr		comptamR		; desvia
		
eh02H:			cp		02H				; compara com 2
				jr		nz,eh03H		; diferente compara outro
				ld		a,T2764			; carrega tamanho a comparar
				jr		comptamR		; desvia
				
eh03H:			cp		03H				; compara com 3
				jr		nz,eherroH		; diferente erro
				ld		a,T27128		; carrega tamanho a comparar
				
comptamR:		cp		d				; compara para ver se chegou ao fim
				jp		nz,read_bytes	; desvia
				ld		a,e				; carrega para comparar
				cp		00H				; compara e desvia para continuar
				jp		nz,read_bytes	; desvia e continua
				
				ld		hl,msgokRead	; insere mensagem no final
				call	wrmonRS232
				
				ret						; retorna

;------------------------------------------------------------------------------
; Subrotina para verificar uma EPROM 
; coloca end nos displays 1 a 4 e dado nos disps de 5 a 6
; também envia informação para tela
;
verif_eprom:	ld		hl,msgCRLF		; insere msg sobre o que vai fazer
				call	wrmonRS232		; envia duas vezes
				ld		hl,msgCRLF		; insere msg sobre o que vai fazer
				call	wrmonRS232

				ld		hl,msgverify	; inicia verificação
				call	wrmonRS232
				
				call	printEPROM		; insere o tipo da eprom
				
				ld		hl,msgCRLF		; endereço e dados na linha debaixo
				call	wrmonRS232
				
				ld		a,CFG8255R		; prepara para "verificar" dados
				call	cfg_8255		; desvia para reprogramar o 8255
				
				ld		a,01H			; prepara porta de controle
				or		EN_OEVPPON		; transistor em condução, pino OE/VPP em ground
				ld		hl,statusB		; pega end da var
				ld		(hl),a			; guarda status
				out		(_8255_OB),a	; port B com controles desligados
				
				ld		d,00H			; carrega DE com o endereço
				ld		e,00H			
				
verify_bytes:	ld		a,d				; d contem MSB do endereço
				out		(_8255_OA),a	; carrega com MSB do endereço
				ld		a,(hl)			; pega o status
				
				or		EN_MSBON		; faz um OR para ligar apenas o pino EN_MSB
				ld		(hl),a			; guarda status
				out		(_8255_OB),a	; transfere MSB do endereço
				
				ld		a,(hl)			; pega o status
				and		EN_MSBOFF		; desliga pino - MSB na posição
				ld		(hl),a			; guarda status
				out		(_8255_OB),a	; desliga pino
				
				ld		a,e				; carrega LSB do endereço
				out		(_8255_OA),a	; carrega com MSB do endereço
				
				ld		a,(hl)			; pega o status
				or		EN_CEON			; pino EN_CE on
				ld		(hl),a			; guarda status
				out		(_8255_OB),a	; liga pino
				
				nop						; aguarda pino estabilizar
				nop
				nop
				nop

				push    hl				; salva regs
				push	af
				ld		hl,epromtype	; pega end da var
				ld		a,(hl)			; conteúdo em A
				cp		01H				; compara com 1
				jr		nz,v64_128on	; desvia
				ld		b,EN_OEVPPON	; carrega b com pino
				jr		contverifON

v64_128on:		ld		b,EN_OEON		; carrega b com pino
contverifON:	pop		af				; restaura regs
				pop		hl
				ld		a,(hl)			; lê status
				
				or		b				; pino EN_OEVPP ou EN_OE
				ld		(hl),a			; guarda status
				out		(_8255_OB),a	; liga pino
				
				nop						; aguarda dado estabilizar
				nop
				nop
				nop
				
				in		a,(_8255_OC)	; pega dado
				ld		c,a				; c agora tem o dado do endereço
				
				push    hl				; salva regs
				push	af
				ld		hl,epromtype	; pega end da var
				ld		a,(hl)			; conteúdo em A
				cp		01H				; compara com 1
				jr		nz,v64_128off	; desvia
				ld		b,EN_OEVPPOFF	; carrega b com pino
				jr		contverifOFF

v64_128off:		ld		b,EN_OEOFF		; carrega b com pino
contverifOFF:	pop		af				; restaura regs
				pop		hl
				
				ld		a,(hl)			; pega o status
				and		b
				and		EN_CEOFF		; pino EN_CE off
				ld		(hl),a			; guarda status
				out		(_8255_OB),a	; desliga pinos CE e OE
				
				;SHOW END
				ld		a,d				; mostra endereço
				call	bin2hex
				ld		a,e
				call	bin2hex
				
				ld		hl,mydataTX		; pega end da var
				ld		a,'H'
				ld		(hl),a			; salba na var
				call	sndchRS232		; envia char
				
				;ld		a,c				; pega o dado
				;call	showdataD		; mostra dado no display
				
				ld		a,c				; pega o dado
				cp		0FFH			; compara com FF
				jr		z,contverif		; continua se igual
				
				ld		hl,msgFound		; envia mensagem
				call	wrmonRS232
				
				ld		a,c				; envia valor
				call	bin2hex
								
				ld		hl,msgerroVerif	; envia mensagem
				call	wrmonRS232
				
				jr		compend
				
				
contverif:		ld		c,BS_
				ld		b,05H
				call	sendCharX		; envia 5 BS_
				
				ld		hl,statusB		; pega end da var
				
				inc		de				; incrementa endereço
				push	hl				; salva
				ld		hl,epromtype	; prepara para comparar
				ld		a,(hl)			; conteúdo em A
				pop		hl				; restaura
eh01V:			cp		01H				; compara com 1
				jr		nz,eh02V		; diferente compara outro
eherroV:		ld		a,T2732			; carrega tamanho a comparar
				jr		comptamV		; desvia
		
eh02V:			cp		02H				; compara com 2
				jr		nz,eh03V		; diferente compara outro
				ld		a,T2764			; carrega tamanho a comparar
				jr		comptamV		; desvia
				
eh03V:			cp		03H				; compara com 3
				jr		nz,eherroV		; diferente erro
	    		ld		a,T27128		; carrega tamanho a comparar
								
comptamV:		cp		d				; compara para ver se chegou ao fim
				jp		nz,verify_bytes	; desvia
				ld		a,e				; carrega para comparar
				cp		00H				; compara e desvia para continuar
				jp		nz,verify_bytes	; desvia e continua
								
				ld		hl,msgokVerif	; envia mensagem
				call	wrmonRS232		; restaura
				
compend:		ret						; retorna				
				
	
;------------------------------------------------------------------------------
; Subrotina para comparar uma EPROM - no modo teste lê apenas 255 bytes
; e coloca end nos displays 1 a 4 e dado nos disps de 5 a 6
; também envia informação para tela
;
compar_eprom:	ld		hl,msgCRLF		; insere msg sobre o que vai fazer
				call	wrmonRS232		; envia duas vezes
				ld		hl,msgCRLF		; insere msg sobre o que vai fazer
				call	wrmonRS232

				ld		hl,msgcompar	; inicia verificação
				call	wrmonRS232
				
				call	printEPROM		; insere o tipo da eprom
				
				ld		hl,msgCRLF		; endereço e dados na linha debaixo
				call	wrmonRS232
				
				ld		a,CFG8255R		; prepara para ler dados
				call	cfg_8255		; desvia para reprogramar o 8255
				
				ld		a,01H			; prepara porta de controle
				or		EN_OEVPPON		; transistor em condução, pino OE/VPP em ground
				ld		hl,statusB		; pega end da var
				ld		(hl),a			; guarda status
				out		(_8255_OB),a	; port B com controles desligados
				
				ld		d,00H			; carrega DE com o endereço
				ld		e,00H	
				
				ld		bc,PONTEPROM	; em BC endereço na RAM para os dados
				
				
compar_bytes:	ld		a,d				; d contem MSB do endereço
				out		(_8255_OA),a	; carrega com MSB do endereço

				ld		a,(hl)			; pega o status
				or		EN_MSBON		; faz um OR para ligar apenas o pino EN_MSB
				ld		(hl),a			; guarda status
				out		(_8255_OB),a	; transfere MSB do endereço
				
				ld		a,(hl)			; pega o status
				and		EN_MSBOFF		; desliga pino - MSB na posição
				ld		(hl),a			; guarda status
				out		(_8255_OB),a	; desliga pino
				
				ld		a,e				; carrega LSB do endereço
				out		(_8255_OA),a	; carrega com MSB do endereço
				
				ld		a,(hl)			; pega o status
				or		EN_CEON			; pino EN_CE on
				ld		(hl),a			; guarda status
				out		(_8255_OB),a	; liga pino
				
				nop						; aguarda pino estabilizar
				nop
				nop
				nop

				push	bc				; salva regs
				push    hl				
				ld		hl,epromtype	; pega end da var
				ld		a,(hl)			; conteúdo em A
				cp		01H				; compara com 1
				jr		nz,c64_128on	; desvia
				
				ld		b,EN_OEVPPON	; carrega b com pino
				jr		contcomparON

c64_128on:		ld		b,EN_OEON		; carrega b com pino
contcomparON:	pop		hl				; restaura reg
				ld		a,(hl)			; lê status
				or		b				; pino EN_OEVPP ou EN_OE
				pop		bc				; restaura reg
				ld		(hl),a			; guarda status
				out		(_8255_OB),a	; liga pino
				
				nop						; aguarda dado estabilizar
				nop
				nop
				nop

				in		a,(_8255_OC)	; pega dado
				push	hl
				ld		hl,aux4			; pega end da var
				ld		(hl),a			; salva
				pop		hl
								
				push	bc				; salva regs
				push    hl				
				ld		hl,epromtype	; pega end da var
				ld		a,(hl)			; conteúdo em A
				cp		01H				; compara com 1
				jr		nz,c64_128off	; desvia
				ld		b,EN_OEVPPOFF	; carrega b com pino
				jr		contcomparOFF

c64_128off:		ld		b,EN_OEOFF		; carrega b com pino
contcomparOFF:	pop		hl				; restaura reg
				ld		a,(hl)			; pega o status
				and		b
				and		EN_CEOFF		; pino EN_CE off
				pop		bc
				ld		(hl),a			; guarda status
				out		(_8255_OB),a	; desliga pinos CE e OE
				
				;SHOW END
				ld		a,d				; mostra endereço
				call	bin2hex
				ld		a,e
				call	bin2hex
				
				ld		hl,mydataTX		; pega end da var
				ld		a,' '
				ld		(hl),a			; salva na var
				call	sndchRS232		; envia char
				call	t_50us
				
				ld		a,(bc)			; pega valor do buffer
				call	bin2hex			; mostra
				
				ld		hl,mydataTX		; pega end da var
				ld		a,'H'
				ld		(hl),a			; salva na var
				call	sndchRS232		; envia char
				call	t_50us
				
				;ld		a,(bc)			; pega o dado
				;call	showdataD		; mostra no display
				
				ld		hl,aux4			; pega end da var
				ld		a,(bc)			; pega o dado
				cp		(hl)			; compara com dado na EPROM
				jr		z,contcompar	; continua se igual
				
				ld		hl,msgFound		; envia mensagem
				call	wrmonRS232
				
				ld		hl,aux4
				ld		a,(hl)
				call	bin2hex			; mostra
				
				ld		hl,mydataTX		; pega end da var
				ld		a,'H'
				ld		(hl),a			; salva na var
				call	sndchRS232		; envia char
				call	t_50us
				
				ld		hl,msgerroCompar; envia mensagem
				call	wrmonRS232
				
				jr		comparend
				
contcompar:		push	bc
				ld		c,BS_
				ld		b,09H
				call	sendCharX		; envia 5 BS_
				pop		bc
				
				ld		hl,statusB		; pega end da var
				
				inc		de				; incrementa endereço
				inc		bc				; incrementa o ponteiro
				push	hl				; salva
				ld		hl,epromtype	; prepara para comparar
				ld		a,(hl)			; conteúdo em A
				pop		hl				; restaura
eh01C:			cp		01H				; compara com 1
				jr		nz,eh02C		; diferente compara outro
eherroC:		ld		a,T2732			; carrega tamanho a comparar
				jr		comptamC		; desvia
		
eh02C:			cp		02H				; compara com 2
				jr		nz,eh03C		; diferente compara outro
				ld		a,T2764			; carrega tamanho a comparar
				jr		comptamC		; desvia
				
eh03C:			cp		03H				; compara com 3
				jr		nz,eherroC		; diferente erro
	    		ld		a,T27128		; carrega tamanho a comparar
								
comptamC:		cp		d				; compara para ver se chegou ao fim
				jp		nz,compar_bytes	; desvia
				ld		a,e				; carrega para comparar
				cp		00H				; compara e desvia para continuar
				jp		nz,compar_bytes	; desvia e continua
				
				ld		hl,msgokCompar	; envia mensagem
				call	wrmonRS232
				
comparend:		ret						; retorna			

;------------------------------------------------------------------------------
; Subrotina para gravar uma EPROM - no modo teste lê apenas 255 bytes
; e coloca end nos displays 1 a 4 e dado nos disps de 5 a 6
; também envia informação para tela
;
write_eprom:	ld		hl,msgCRLF		; insere msg sobre o que vai fazer
				call	wrmonRS232		; envia duas vezes
				ld		hl,msgCRLF		; insere msg sobre o que vai fazer
				call	wrmonRS232

				ld		hl,msgwrite	; inicia verificação
				call	wrmonRS232
				
				call	printEPROM		; insere o tipo da eprom
				
				ld		hl,msgCRLF		; endereço e dados na linha debaixo
				call	wrmonRS232
				
				ld		a,CFG8255W		; prepara para gravar dados
				call	cfg_8255		; desvia para reprogramar o 8255
				
				ld		a,01H			; prepara porta de controle
				or		EN_OEVPPON		; transistor em condução, pino OE/VPP em ground
				ld		hl,statusB		; pega end da var
				ld		(hl),a			; guarda status
				out		(_8255_OB),a	; port B com controles desligados
				
				ld		d,00H			; carrega DE com o endereço
				ld		e,00H	
				
				ld		bc,PONTEPROM	; em BC endereço na RAM para os dados
				
write_bytes:	;SHOW END DATA
				ld		a,d				; mostra endereço
				call	bin2hex
				ld		a,e
				call	bin2hex
				
				ld		hl,mydataTX		; pega end da var
				ld		a,'H'
				ld		(hl),a			; salva na var
				call	sndchRS232		; envia char
				
				ld		hl,mydataTX		; pega end da var
				ld		a,' '
				ld		(hl),a			; salva na var
				call	sndchRS232		; envia char
				
				ld		a,(bc)			; pega dado
				call	bin2hex			; mostra na tela
				ld		hl,mydataTX		; pega end da var
				ld		a,'H'
				ld		(hl),a			; salva na var
				call	sndchRS232		; envia char
				
				;ld		a,(bc)			; pega o dado
				;call	showdataD		; mostra no display
				
				ld		a,d				; d contem MSB do endereço
				out		(_8255_OA),a	; carrega com MSB do endereço

				ld		hl,statusB		; pega end da var
				ld		a,(hl)			; pega o status
				or		EN_MSBON		; faz um OR para ligar apenas o pino EN_MSB
				ld		(hl),a			; guarda status
				out		(_8255_OB),a	; transfere MSB do endereço
				
				ld		a,(hl)			; pega o status
				and		EN_MSBOFF		; desliga pino - MSB na posição
				ld		(hl),a			; guarda status
				out		(_8255_OB),a	; desliga pino
				
				ld		a,e				; carrega LSB do endereço
				out		(_8255_OA),a	; carrega com MSB do endereço

				ld		a,(bc)			; pega byte a gravar
				out		(_8255_OC),a	; carrega no port
				
				nop						; aguarda estabilizar
				nop
				
				push	bc
				push    hl				; 
				ld		hl,epromtype	; pega end da var
				ld		a,(hl)			; conteúdo em A
				cp		01H				; compara com 1
				jr		nz,w64_128		; desvia
				pop		hl
				pop		bc
				and		EN_OEVPPOFF		; coloca VPP em OEVPP
				ld		(hl),a			; guarda status
				out		(_8255_OB),a	; liga pino
				call	t_1ms
				
				
				ld		hl,statusB		; pega end da var
				ld		a,(hl)			; pega o status
				or		EN_CEON			; pino EN_CE on
				ld		(hl),a			; guarda status
				out		(_8255_OB),a	; liga pino
				
				call	t_50ms			; aguarda 50ms
				call	t_5ms			; aguarda +5ms x 2
				call	t_5ms

				ld		hl,statusB		; pega end da var
				ld		a,(hl)			; pega o status
				and		EN_CEOFF		; pino EN_CE off
				ld		(hl),a			; guarda status
				out		(_8255_OB),a	; desliga pino CE
				call	t_1ms
								
				ld		hl,statusB		; pega end da var
				ld		a,(hl)			; pega o status
				or		EN_OEVPPON		; remove VPP de OEVPP
				ld		(hl),a			; guarda status
				out		(_8255_OB),a	; liga pino
				jr		contwrite		; desvia para continuar
				
w64_128:		pop		hl
				pop		bc
				ld		a,(hl)			; pega o status
				or		EN_CEON			; pino EN_CE on
				ld		(hl),a			; guarda status
				out		(_8255_OB),a	; liga pino
				nop						; 2us
				nop
				and		EN_OEVPPOFF		; coloca VPP em VPPpin
				ld		(hl),a			; guarda status
				out		(_8255_OB),a	; liga pino
				nop
				nop
				or		EN_PGMON		; liga pino PGM
				ld		(hl),a			; guarda status
				out		(_8255_OB),a	; liga pino
				
				call	t_1ms           ; aguarda
				call	t_50us
				
				and		EN_PGMOFF		; desliga pino PGM
				ld		(hl),a			; guarda status
				out		(_8255_OB),a	; liga pino
				nop
				nop
				or		EN_OEVPPON		; desliga VPP em VPPpin
				ld		(hl),a			; guarda status
				out		(_8255_OB),a	; liga pino
				nop
				nop
						
contwrite:		push	bc
				ld		c,BS_
				ld		b,09H
				call	sendCharX		; envia 09 BS_
				pop		bc
				
				inc		de				; incrementa endereço
				inc		bc				; incrementa ponteiro
				push	hl				; salva
				ld		hl,epromtype	; prepara para comparar
				ld		a,(hl)			; conteúdo em A
				pop		hl				; restaura
eh01W:			cp		01H				; compara com 1
				jr		nz,eh02W		; diferente compara outro
eherroW:		ld		a,T2732			; carrega tamanho a comparar
				jr		comptamW		; desvia
		
eh02W:			cp		02H				; compara com 2
				jr		nz,eh03W		; diferente compara outro
				ld		a,T2764			; carrega tamanho a comparar
				jr		comptamW		; desvia
				
eh03W:			cp		03H				; compara com 3
				jr		nz,eherroW		; diferente erro
	    		ld		a,T27128		; carrega tamanho a comparar
								
comptamW:		cp		d				; compara para ver se chegou ao fim
				jp		nz,write_bytes	; desvia
				ld		a,e				; carrega para comparar
				cp		00H				; compara e desvia para continuar
				jp		nz,write_bytes	; desvia e continua
				
								
writeend:		ld		hl,msgokWrite	; envia msg
				call	wrmonRS232		
				
				ld		a,CFG8255R		; prepara para ler dados
				call	cfg_8255		; desvia para reprogramar o 8255
				
				ret						; retorna			
				

;-------------------------------------------------------------------------------
; Subrotina para mostrar o dado
; insere resultado nos displays
; em DE endereço do byte, em A o dado
;
showdataD:		ld		hl,aux			; var auxiliar
				ld		(hl),b			; salva b na var auxiliar
				
				ld		hl,aux3			; var auxiliar
				ld		(hl),c			; salva c na var auxiliar
				
				ld		c,a				; salva o dado em c
				
				ld		a,d				; pega MSB do endereço
				and		0F0H			; limpa lsb do byte
				rra						; gira 4 vezes
				rra
				rra
				rra
				ld		b,01H			; informa valor para display 1
				call	wrAdisp1_6
				
				ld		a,d				; pega MSB do endereço
				and		0FH				; limpa MSB do byte
				ld		b,02H			; informa valor para display 2
				call	wrAdisp1_6
				
				ld		a,e				; pega LSB do endereço
				and		0F0H			; limpa lsb do byte
				rra						; gira 4 vezes
				rra
				rra
				rra
				ld		b,03H			; informa valor para display 3
				call	wrAdisp1_6
				
				ld		a,e				; pega LSB do endereço
				and		0FH				; limpa MSB do byte
				ld		b,04H			; informa valor para display 4
				call	wrAdisp1_6

				ld		a,c				; pega o dado
				call	wrAdisp5_6		; e coloca nos disps 5 e 6
				
				ld		hl,aux			; pega end da var auxiliar
				ld		b,(hl)			; restaura b da var auxiliar
				
				ld		hl,aux3			; pega end da var auxiliar
				ld		c,(hl)			; restaura c da var auxiliar
				
				ret						; retorna
		
;-------------------------------------------------------------------------------
; Subrotina para mostrar o dado
; via RS232 - em A o dado
;
showdataS:		call	bin2hex
				
				ld		hl,mydataTX		; pega end da var
				ld		a,' '
				ld		(hl),a			; salba na var
				call	sndchRS232		; envia char
				
				ld		hl,aux2			; pega end da var
				inc		(hl)			; incrementa conteúdo
				ld		a,(hl)			; carrega A
				cp		10H				; compara com 16
				jr		nz,notsndCRLF	; diferente, continua
				
sndCRLF:		call	myCRLF
				
				inc		de				; incrementa apenas para ajustar
				ld		a,d				; envia endereço
				call	bin2hex			; MSB
				ld		a,e
				call	bin2hex			; LSB
				dec		de				; decrementa para corrigir
								
				ld		hl,mydataTX		; envia espaço para alinhar
				ld		a,' '
				ld		(hl),a
				call	sndchRS232
				
sndesp:			ld		hl,aux2			; zera para iniciar próximas colunas
				ld		a,00H
				ld		(hl),a
				
notsndCRLF:		ret						; retorna

;------------------------------------------------------------------------------
; Subrotina para enviar para tela o tipo da EPROM
;

printEPROM:		ld		hl,epromtype	; insere perfumaria
				ld		a,(hl)
				
eh01L:			cp		01H				; compara com 1
				jr		nz,eh02L		; desvia para próximo
eherroL:		ld		hl,msg2732
				call	wrmonRS232
				jr		printEend		; desvia para finalizar
				
eh02L:			cp		02H				; compara com 2
				jr		nz,eh03L		; desvia para próximo
				ld		hl,msg2764
				call	wrmonRS232
				jr		printEend
				
eh03L:			cp		03H
				jr		nz,eherroL
				ld		hl,msg27128
				call	wrmonRS232
									
printEend:		ret

;-------------------------------------------------------------------------------
; Subrotina para imprimir um dado na tela em formato hexadecimal
; Parametros: em A valor do dado a ser convertido!
;
bin2hex:        push	hl
				push    bc
				push	de
                push    af

                and     0F0H            ;separa apenas mais significativo
                rra                     ;gira para colocar MSB no LSB
                rra                     ;sao 4 bits
                rra
                rra
                and     0FH             ;limpa o MSB
                ld      c,a
                ld      b,00H
                ld      hl,_BCD         ;pega endereco do BDC
                add     hl,bc           ;soma o ponteiro
                ld      a,(hl)
                ld      hl,mydataTX     ;pega endereço da var de envio
                ld      (hl),a          ;carrega var com a
                call    sndchRS232      ;envia caracter
                ld		d,0
                ld		e,1
                call    wait_ms        ;tempo

                pop     af              ;restaura

                and     0FH             ;separa apenas menos significativo
                ld      c,a
                ld      b,00H
                ld      hl,_BCD         ;pega endereco do BDC
                add     hl,bc           ;soma o ponteiro
                ld      a,(hl)
                ld      hl,mydataTX     ;pega endereço da var de envio
                ld      (hl),a          ;carrega var com a
                call    sndchRS232      ;envia dado
                ld		d,0
                ld		e,1
                call    wait_ms        ;tempo

                pop		de
                pop     bc
                pop		hl
                ret                     ;retorna
           
;-------------------------------------------------------------------------------
; Subrotina para enviar via RS232 o mesmo char, n vezes
; em C o char e em B a quantidade de vezes (255 máximos)
;
sendCharX:		ld		hl,mydataTX		; pega end da var
				ld		(hl),c			; salva na var
				call	sndchRS232		; envia char
				call	t_50us			; temporiza
				djnz	sendCharX		; decrementa b e continua
				
				ret						; retorna
				
;-------------------------------------------------------------------------------
; Subrotina para enviar ENTER para o terminal
;
myCRLF:			push	de
				ld		hl,msgCRLF
				call	wrmonRS232
				pop		de
				
				ret
				
;-------------------------------------------------------------------------------
; Subrotina para configurar a 8255
; byte de configuração em A
;
cfg_8255:       out     (_8255_CFG),a   ;envia para 8255 palavra de config
                ret   
                            
;------------------------------------------------------------------------------
; Subrotina para aguardar x ms por espera ocupada
; em de o valor a aguardar - 1 a 65535
; clock de 3,575611MHz, então T = 279,7ns (1T = 279,7ns)
;
wait_ms:		push	af				; salva registradores
				push	de
				push    hl				; 3 x 11T = 9,229us
				
								
loopxms:        ld      h,05H           ; 7T = 1,957us
loop1ms:        ld      l,2BH           ; 7T = 1,957us
loop1_1ms:      dec     l               ; 4T = 1,118us
                jr      nz,loop1_1ms    ; 12T (3,356us) x l + 7T (1,957us)
                dec     h               ; 4T = 1,118us
                jr      nz,loop1ms      ; 12T (3,356us) x h + 7T (1,957us)
				
				nop						; 4T = 1,118us
				
				dec		de				; 6T = 1,678us
				ld		a,d				; 9T = 2,517us
				or		e				; 4T = 1,118us
				jp		nz,loopxms		; 12T (3,356us) x de + 7T (1,957us)
                
                nop						; 4T = 1,118us
                nop						; 4T = 1,118us
                
                pop     hl				; 3 x 10T = 2,796us
				pop		de
				pop		af
				
				ret						; retorna
			
;------------------------------------------------------------------------------
; mensagens
;
msginit:		db		CR,LF,CR,LF
				db		'Programador EPROM p/ placa Z80 1990 by Arne'
				db		CR,LF,00H
				
msgread:		db		'Lendo Dados (',00H
msgread2:		db		CR,LF,'     00 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F'
msgCRLFESP:		db		CR,LF,'0000 ',00H

msgokRead:		db		CR,LF,'Pronto! Dados no Buffer!',00H

msgverify:		db		'Verificando Dados (',00H
msgcompar:		db		'Comparando Dados (',00H
msgwrite:		db		'Gravando Dados (',00H

msgCRLF:		db		CR,LF,00H

msgend:			db		CR,LF,CR,LF
				db		'Fim',00H
				
				
msgnotimp:		db		'Nao implementado!',00H
msgepinv:		db		'Eprom invalida!',00H
msgerroOp:		db		'Opcao invalida!',00H

msgmenuop:		db		CR,LF,'Opcoes (',00H
msgmenuop2:		db		CR,LF,CR,LF,'1 - Ler'
				db		CR,LF,'2 - Verificar'
				db		CR,LF,'3 - Comparar'
				db		CR,LF,'4 - Gravar',00H
				db		CR,LF,'5 - Voltar',00H
				
msgmenuR:		db		CR,LF,CR,LF,'Entre com sua opcao: ',00H
				
msgmenuEPROM:	db		CR,LF,'Tipo de EPROM:'
				db		CR,LF,CR,LF,'1 - 2732'
				db		CR,LF,'2 - 2764'
				db		CR,LF,'3 - 27128',00H
				db		CR,LF,'4 - Sair',00H
				
msg2732:		db		'2732): ',00H
msg2764:		db		'2764): ',00H
msg27128:		db		'27128): ',00H

msgEPROM:		db		'EPROM ',00H

msgokVerif:		db		CR,LF,'EPROM vazia!',00H
msgerroVerif:	db		' EPROM nao vazia!',00H

msgokWrite:		db		CR,LF,'EPROM gravada!',00H
msgFound:		db		' encontrado -> ',00H

msgokCompar:	db		CR,LF,'EPROM igual!',00H
msgerroCompar:	db		' EPROM diferente!',00H

msgerroEnd:		db		'End -> ',00H

;-------------------------------------------------------------------------------
; Constantes do modulo
;
_BCD:           db  30H, 31H, 32H, 33H, 34H, 35H, 36H, 37H
                db  38H, 39H, 41H, 42H, 43H, 44H, 45H, 46H,00H 
                
statusB         db      1               ; var para guardar o status do portB
aux             db      1               ; vars auxiliares
aux2            db      1
aux3			db		1
aux4			db		1
menu			db		1				; guarda opções do menu
epromtype		db		1				; guarda o tipo da eprom
PONTEPROM		db		1				; ponteiro na memória RAM para conter o prog

;------------------------------------------------------------------------------
; Fim do programa
;
				end
